--***************************************************************
--* File         : TB_TOP.vhd                                   *
--* Created      : time-stamp: <08/05/22 17:31:20 kawasan>      *
--* Modified     : Time-stamp: <08/06/13 02:35:26 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2008 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_TOP is end;

architecture behavior of TB_TOP is
    
	component TOP
	generic (
		CLOCK_TYPE      : string  := "DEFAULT";			-- DEFAULT or ACTEL_CLKINT
		INSERTWAITST    : boolean := FALSE;
		ROMWIDTH        : positive := 10;
		RAMWIDTH        : positive := 12;
		RAMSIZE         : positive := 2**12
	);
	port (
		RESET_N         : in    std_logic;	-- active logic '0'
		SYSCLK          : in    std_logic;

		-- PORT
		PORTA           : inout std_logic_vector(7 downto 0);
		PORTB           : inout std_logic_vector(7 downto 0);

		-- UART 
		RXD             : in    std_logic;
		TXD             : out   std_logic;

		-- debug
		DBG             : out   std_logic_vector(7 downto 0)
	);
	end component;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant CLOCK_TYPE     : string  := "DEFAULT";
	constant INSERTWAITST   : boolean := FALSE;
	constant ROMWIDTH       : positive := 10;
	constant RAMWIDTH       : positive := 12;
	constant RAMSIZE        : positive := 2**RAMWIDTH;

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal RESET_N          : std_logic := '0';
	signal SYSCLK           : std_logic := '0';
	signal PORTA            : std_logic_vector(7 downto 0) := (others=>'0');
	signal PORTB            : std_logic_vector(7 downto 0) := (others=>'0');
	signal RXD              : std_logic := '0';
	signal TXD              : std_logic := '0';
	signal DBG              : std_logic_vector(7 downto 0) := (others=>'0');

begin
    
	uTOP : TOP
	generic map (
		CLOCK_TYPE      => CLOCK_TYPE,
		INSERTWAITST    => INSERTWAITST,
		ROMWIDTH        => ROMWIDTH,
		RAMWIDTH        => RAMWIDTH,
		RAMSIZE         => RAMSIZE
	)
	port map (
		RESET_N         => RESET_N,
		SYSCLK          => SYSCLK,

		-- PORT
		PORTA           => PORTA,
		PORTB           => PORTB,

		-- UART 
		RXD             => RXD,
		TXD             => TXD,

		-- debug
		DBG             => DBG
	);

	RXD <= TXD;

	process begin
		SYSCLK <= '0';
		wait for CLK_CYCLE/2;
		SYSCLK <= '1';
		wait for CLK_CYCLE/2;
	end process;

	process
		variable ln         : line;
	begin
		-- Init
		wait for WT_IDLE+WT_DELAY;
		RESET_N <= '1';

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

end behavior;
